"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.mdPluginVuePreview = void 0;
const markdown_1 = require("@vuepress/markdown");
const utils_1 = require("@vuepress/utils");
const doc_vue3_1 = require("doc-vue3");
const analyze_deps_1 = require("./analyze-deps");
const md_instance_1 = require("./md-instance");
const md = (0, markdown_1.createMarkdown)();
(0, md_instance_1.highlightCode)(md);
let hasImportBlockOpen = false;
let importMode = '';
let componentName = null;
let importBlockIndex = 0;
let resolveFileError = false;
const root = process.cwd();
function getAbsPath(path) {
    return path.trim().replace(/^@/, root).trim();
}
const mdPluginVuePreview = function (md) {
    // 覆盖块标签-起始标签
    md.renderer.rules.paragraph_open = function (tokens, idx, options, env, self) {
        const contentToken = tokens[idx + 1];
        const matchImportPattern = contentToken.type === 'inline' &&
            contentToken.content.match(/^@\[(preview|docvue)-?(\w+)?\]\((.+)\)/);
        if (!matchImportPattern) {
            return self.renderToken(tokens, idx, options);
        }
        importMode = matchImportPattern[1];
        componentName = matchImportPattern[2];
        hasImportBlockOpen = true;
        importBlockIndex = idx;
        const filePath = matchImportPattern[3];
        const absoluteFilePath = getAbsPath(filePath);
        if (!utils_1.fs.existsSync(absoluteFilePath)) {
            resolveFileError = true;
            return `<div class="custom-container warning"><p>未找到文件: ${absoluteFilePath}</p></div><!-- `;
        }
        if (!/\.vue$/.test(absoluteFilePath)) {
            resolveFileError = true;
            return `<div class="custom-container warning"><p>不支持非 vue 文件: ${filePath}</p></div><!-- `;
        }
        resolveFileError = false;
        if (importMode === 'preview') {
            return renderDemoOpen({
                filePath,
                absoluteFilePath,
                showDemo: componentName,
            });
        }
        if (importMode === 'docvue') {
            const content = utils_1.fs.readFileSync(absoluteFilePath, 'utf-8');
            const htmlResult = (0, doc_vue3_1.docVue3)(content || '', { type: 'html' });
            return htmlResult + '<!-- ';
        }
    };
    // 覆盖块标签-结束标签
    md.renderer.rules.paragraph_close = function (tokens, idx, options, env, self) {
        if (importMode === 'docvue') {
            return ' -->';
        }
        if (hasImportBlockOpen && importBlockIndex + 2 === idx) {
            hasImportBlockOpen = false;
            return !resolveFileError
                ? '--></DenraoCodeGroup></DenraoVuePreview>'
                : ' -->';
        }
        return self.renderToken(tokens, idx, options);
    };
    // 移除段落标签
    // md.renderer.rules.paragraph_close = md.renderer.rules.paragraph_open = doNotRenderTokenWhenBlockOpen
};
exports.mdPluginVuePreview = mdPluginVuePreview;
function renderDemoOpen({ filePath, absoluteFilePath, showDemo }) {
    const template = `<DenraoVuePreview absoluteFilePath="${absoluteFilePath}" showDemo="${showDemo}"><DenraoCodeGroup>`;
    const deps = (0, analyze_deps_1.analyzeDeps)(absoluteFilePath);
    const codeGroups = `${[absoluteFilePath]
        .concat(deps)
        .map((absPath, index) => {
        return `<DenraoCodeGroupItem title="${utils_1.path.basename(absPath)}">
      ${md.render(`@[code](${absPath})`)}
      </DenraoCodeGroupItem>`;
    })
        .join('')}`;
    return template + codeGroups + '' + '<!-- ';
}
